import oneai
import pytest
from tests.constants import DOCUMENT, URL_INPUT


def test_custom_text():
    pipeline = oneai.Pipeline(
        [
            oneai.Skill("enhance", text_attr="custom_text"),
        ]
    )
    output = pipeline.run(DOCUMENT)
    assert (
        hasattr(output, "custom_text")
        and output.custom_text is not None
        and hasattr(output.custom_text, "text")
        and output.custom_text.text is not None
    )


def test_custom_labels():
    pipeline = oneai.Pipeline(
        [
            oneai.Skill("enhance", labels_attr="custom_labels"),
        ]
    )
    output = pipeline.run(DOCUMENT)
    assert hasattr(output, "custom_labels") and output.custom_labels is not None


def test_custom_text_and_labels():
    pipeline = oneai.Pipeline(
        [
            oneai.Skill(
                "enhance", text_attr="custom_text", labels_attr="custom_labels"
            ),
        ]
    )
    output = pipeline.run(DOCUMENT)
    assert hasattr(output, "custom_text") and output.custom_text is not None
    assert (
        hasattr(output.custom_text, "custom_labels")
        and output.custom_text.custom_labels is not None
    )


def test_custom_within_pipeline():
    pipeline = oneai.Pipeline(
        [
            oneai.skills.Summarize(),
            oneai.Skill(
                "enhance", text_attr="custom_text", labels_attr="custom_labels"
            ),
            oneai.skills.Proofread(),
            oneai.skills.Anonymize(),
        ]
    )
    output = pipeline.run(DOCUMENT)
    assert hasattr(output.summary, "custom_text")
    assert hasattr(output.summary.custom_text, "text")
    assert hasattr(output.summary.custom_text, "custom_labels")
    assert hasattr(output.summary.custom_text, "proofread")
    assert hasattr(output.summary.custom_text.proofread, "replacements")


def test_modify_predefined_skill():
    pipeline = oneai.Pipeline([oneai.skills.HtmlToArticle(labels_attr="magic_labels")])
    output = pipeline.run(URL_INPUT)
    assert (
        hasattr(output.html_article, "magic_labels")
        and output.html_article.magic_labels
    )


def test_init_skill():
    attrs = ["prompt_fields", "prompt", "temperature"]
    skill = oneai.skills.GPT()
    assert all(hasattr(skill, a) for a in attrs)
    # should not crash - allow params not defined explicitly in SDK
    with pytest.warns(UserWarning):
        skill.fake_param = "fake"



import oneai
import pytest

from tests.constants import MP3_PATH, WAV_PATH, PDF_PATH

pipeline_audio = oneai.Pipeline(
    [
        oneai.skills.Transcribe(timestamp_per_word=True),
        oneai.skills.SplitBySentence(),
        oneai.skills.SplitByTopic(),
        oneai.skills.Proofread(),
        oneai.skills.Numbers(),
        oneai.skills.Sentiments(),
    ]
)


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "path, sync", [(MP3_PATH, False), (MP3_PATH, True), (WAV_PATH, True)]
)
async def test_audio(path: str, sync: bool):
    with open(path, "rb") as f:
        output = pipeline_audio.run(f) if sync else await pipeline_audio.run_async(f)
        assert hasattr(output, "transcription")
        assert hasattr(output.transcription, "sentences")
        assert hasattr(output.transcription, "segments")
        assert hasattr(output.transcription, "proofread")
        assert hasattr(output.transcription.proofread, "replacements")
        assert hasattr(output.transcription.proofread, "numbers")
        assert hasattr(output.transcription.proofread, "sentiments")


@pytest.mark.asyncio
@pytest.mark.parametrize("sync", [False, True])
async def test_pdf(sync: bool):
    if sync:
        pytest.skip("PDF extraction is not supported in sync mode")
    pipeline_pdf = oneai.Pipeline([oneai.skills.PDFExtractText()])
    with open(PDF_PATH, "rb") as f:
        output = pipeline_pdf.run(f) if sync else await pipeline_pdf.run_async(f)
        assert hasattr(output, "pdf_text")
        assert output.pdf_text.text



import pytest
import oneai

collection = oneai.clustering.Collection("test-collection", api_key=oneai.api_key)
cluster: oneai.clustering.Cluster = None
phrase: oneai.clustering.Phrase = None

items = [
    "Can not access account",
    "I cannot access my account",
    "Can not enter my account",
    "Cancel order",
    "I want to cancel my order",
    "How do I cancel my order?",
    "Need help cancelling order",
]


def test_add_items():
    assert "status" in collection.add_items(items)

    # insert via pipeline
    oneai.Pipeline(
        steps=[oneai.skills.CollectionInsert(collection=collection.id)]
    ).run_batch(items)


def test_get_collections():
    collec = next(oneai.clustering.get_collections(limit=1, api_key=oneai.api_key))
    assert collec.id


@pytest.mark.dependency()
def test_get_clusters():
    global cluster

    cluster = next(collection.get_clusters(limit=1))
    assert cluster.collection


@pytest.mark.dependency(depends=["test_get_clusters"])
def test_get_phrases():
    global phrase

    phrase = next(cluster.get_phrases(limit=1))
    assert phrase.cluster


@pytest.mark.dependency(depends=["test_get_phrases"])
def test_get_items():
    item: oneai.clustering.Item = next(phrase.get_items(limit=1))
    assert item.phrase


def test_empty_collection():
    assert (
        next(
            oneai.clustering.Collection(
                "empty-collection-keep-empty", api_key=oneai.api_key
            ).get_clusters(),
            None,
        )
        == None
    )


def test_find_clusters():
    matches = [cluster.text for cluster in collection.find_clusters("access account")]
    assert "Can not access account" in matches


def test_find_phrases():
    matches = [
        phrase.text
        for phrase in collection.find_phrases("cancel order", include_items=True)
    ]
    assert "Cancel order" in matches

    # find via pipeline
    assert (
        oneai.Pipeline(steps=[oneai.skills.CollectionSearch(collection=collection.id)])
        .run("cancel order")
        .matches
    )



from typing import List
import oneai
import pytest

from tests.constants import DOCUMENT, CONVERSATION, URL_INPUT
from tests.util import hasattrnested


@pytest.mark.parametrize(
    "pipeline, input, attrs, notes",
    [
        (
            oneai.Pipeline([oneai.skills.Keywords()]),
            DOCUMENT,
            ["keywords"],
            "simple document",
        ),
        (
            oneai.Pipeline([oneai.skills.Keywords()]),
            CONVERSATION,
            ["keywords"],
            "simple conversation",
        ),
        (
            oneai.Pipeline([oneai.skills.Summarize(), oneai.skills.Keywords()]),
            DOCUMENT,
            ["summary", "summary.keywords"],
            "first Skill generator",
        ),
        (
            oneai.Pipeline(
                [
                    oneai.skills.Emotions(),
                    oneai.skills.Summarize(),
                    oneai.skills.Keywords(),
                ]
            ),
            DOCUMENT,
            ["emotions", "summary", "summary.keywords"],
            "empty labels, then generator",
        ),
        (
            oneai.Pipeline(
                [
                    oneai.skills.Summarize(),
                    oneai.skills.Proofread(),
                    oneai.skills.Anonymize(),
                ]
            ),
            DOCUMENT,
            ["summary", "summary.proofread", "summary.proofread.anonymized"],
            "multiple generators",
        ),
        (
            oneai.Pipeline(
                [
                    oneai.Skill("summarize", text_attr="custom_summary"),
                ]
            ),
            DOCUMENT,
            ["custom_summary"],
            "custom Skill only",
        ),
        (
            oneai.Pipeline(
                [
                    oneai.skills.Summarize(),
                    oneai.skills.Names(),
                    oneai.Skill("keywords", labels_attr="custom_keywords"),
                    oneai.skills.Keywords(),
                    oneai.skills.Summarize(),
                ]
            ),
            DOCUMENT,
            [
                "summary",
                "summary.names",
                "summary.custom_keywords",
                "summary.keywords",
                "summary.summary",
            ],
            "custom Skill, surrounded by core Skills",
        ),
        (
            oneai.Pipeline(
                [
                    oneai.skills.HtmlToArticle(),
                    oneai.skills.Emotions(),
                ]
            ),
            URL_INPUT,
            ["html_article", "html_article.html_labels", "html_article.emotions"],
            "URL input",
        ),
    ],
)
def test_pipeline(
    pipeline: oneai.Pipeline, input: oneai.TextContent, attrs: List[str], notes: str
):
    try:
        output = pipeline.run(input)
        for attr in attrs:
            assert hasattrnested(output, attr)
    except Exception as e:
        pytest.fail(f"Pipeline failed on {notes}: {e}")


def test_batch():
    inputs = [
        DOCUMENT,
        CONVERSATION,
        DOCUMENT,
        CONVERSATION,
        URL_INPUT,
    ]
    outputs = oneai.Pipeline(
        [oneai.skills.Names(), oneai.skills.Summarize(), oneai.skills.Keywords()]
    ).run_batch(inputs)

    for input in inputs:
        assert input in outputs
        assert isinstance(
            outputs[input], Exception if input == URL_INPUT else oneai.Output
        )


def test_generate_chapters():
    assert oneai.util.generate_chapters(DOCUMENT, amount="more")
